#pragma once

#include <functional>
#include <map>
#include <string>

namespace rei::json {

	class JsonArray;
	class FieldValue;

	class JsonObject {
		public:

		JsonObject();

		bool contains(const std::string& key) const;

		JsonObject& set(const std::string& key, int value);
		JsonObject& set(const std::string& key, bool value);
		JsonObject& set(const std::string& key, const std::string& value);
		JsonObject& set(const std::string& key, const std::string&& value);
		JsonObject& set(const std::string& key, const char * value);
		JsonObject& setNull(const std::string& key);
		JsonObject& addArray(const std::string& key, const JsonArray& array);
		JsonObject& addArray(const std::string& key, JsonArray&& array);
		JsonObject& addObject(const std::string& key, const JsonObject& object);
		JsonObject& addObject(const std::string& key, JsonObject&& object);

		int getNumber(const std::string& key);
		bool getBool(const std::string& key);
		std::string getString(const std::string& key);
		JsonObject& getObject(const std::string& key);
		JsonArray& getArray(const std::string& key);
		bool hasNull(const std::string& key);

		void forEach(std::function<void(FieldValue&, const std::string& key)>);
		size_t count() const;

		private:

		std::map<std::string, FieldValue> elements;
	};
}
